      program fake_adi
!
      use numerics
      implicit none
      integer, parameter  ::  ldx = 129
      integer, parameter  ::  ldy = 129
      integer, parameter  ::  ldz = 128
      integer, parameter  ::  nx  = 128
      integer, parameter  ::  ny  = 128
      integer, parameter  ::  nz  = 128
      integer, parameter  ::  maxsteps = 2
!
      real(rp), dimension(ldx, ldy, ldz)  ::  data
      real(rp), dimension(ldx, ldz)  ::  temp
!
      integer  ::  i, j, k, istep
      real(rp)   ::  checksum
      real(rp), external  ::  rand
      real(sp), external  ::  dtime
      real(sp)  ::  t, t2(2)
!
      do k = 1, nz
         do j = 1, ny
            do i = 1, nx
               data(i,j,k) = rand()
            end do
         end do
      end do
!
      t = dtime(t2)
!
      do istep = 1, maxsteps
!
         do k = 1, nz
            do j = 1, ny
               call xsweep(data(1,j,k),1,nx)
            end do
         end do
!
         do k = 1, nz
            do i = 1, nx
               call ysweep(data(i,1,k),ldx,ny)
            end do
         end do
!
      do j = 1, ny
            call copy(data(1,j,1),ldx*ldy,temp,nx,nx,nz)
            do i = 1, nx
               call zsweep(temp(i,1),nx,nz)
            end do
            call copy(temp,nx,data(1,j,1),ldx*ldy,nx,nz)
         end do
      end do
!
!     do j = 1, ny
!           do i = 1, nx
!              call zsweep(data(i,j,1),ldx*ldy,nz)
!           enddo
!        enddo
!     enddo
!
      t = dtime(t2)
      write(6,1) t
 1    format(1x,'Time:     ',f6.3,' seconds')
      checksum = 0.0d0
      do k = 1, nz
         do j = 1, ny
            do i = 1, nx
               checksum = checksum + data(i,j,k)
            end do
         end do
      end do
!
      write(6,2) checksum
 2    format(1x,'Checksum: ',1pe17.10)
!
      end program fake_adi
!---------------------------------------------------------------------
      subroutine xsweep(v,is,n)
!
      use numerics
      implicit none
      real(rp), dimension(1+is*(n-1)) ::  v
      integer  ::  is, n
      integer  ::  i
      real(rp), parameter  ::  half = 0.5D0
!
      do i = 2, n
         v(1+is*(i-1)) = v(1+is*(i-1)) + half*v(1+is*(i-2))
      end do
!
      do i = n-1, 1, -1
         v(1+is*(i-1)) = v(1+is*(i-1)) - half*v(1+is*i)
      end do
!
      return
      end subroutine xsweep
!---------------------------------------------------------------------
      subroutine ysweep(v,is,n)
!
      use numerics
      implicit none
      real(rp), dimension(1+is*(n-1)) ::  v
      integer  ::  is, n
      integer  ::  i
      real(rp), parameter  ::  half = 0.5D0
!
      do i = 2, n
         v(1+is*(i-1)) = v(1+is*(i-1)) + half*v(1+is*(i-2))
      end do
!
      do i = n-1, 1, -1
         v(1+is*(i-1)) = v(1+is*(i-1)) - half*v(1+is*i)
      end do
!
      return
      end subroutine ysweep
!---------------------------------------------------------------------
      subroutine zsweep(v,is,n)
!
      use numerics
      implicit none
      real(rp), dimension(1+is*(n-1)) ::  v
      integer  ::  is, n
      integer  ::  i
      real(rp), parameter  ::  half = 0.5D0
!
      do i = 2, n
         v(1+is*(i-1)) = v(1+is*(i-1)) + half*v(1+is*(i-2))
      end do
!
      do i = n-1, 1, -1
         v(1+is*(i-1)) = v(1+is*(i-1)) - half*v(1+is*i)
      end do
!
      return
      end subroutine zsweep
!---------------------------------------------------------------------
      subroutine copy(from,lf,to,lt,nr,nc)
!
      use numerics
      implicit none
      real(rp), dimension(lf, nc)  ::  from
      real(rp), dimension(lt, nc)  ::  to
      integer  ::  lf, lt, nr, nc
      integer  ::  i, j
      do j = 1, nc
         do i = 1, nr
            to(i,j) = from(i,j)
         end do
      end do
!
      return
      end subroutine copy
